#!/usr/bin/perl

use strict;
use warnings;

my $dataFile = "position.log";
my $conf_fopenRetry = 3;
my $conf_retrySleep = 0.5;

# add file path for IIS
if( $0 =~ /^[A-Z]:\\/ ) {
  my $fp = $ENV{'SCRIPT_NAME'};
  $fp =~ s/^\/(.+)\/.+$/$1/g;
  $dataFile = $fp . "/${dataFile}";
}

my $buffer;
my $res_mode;
if ($ENV{'REQUEST_METHOD'} eq "POST") {
    read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
} else { $buffer = $ENV{'QUERY_STRING'}; }

my %FORM;
my @pairs = split(/&/, $buffer);
foreach my $pair (@pairs) {
    (my $name, my $value) = split(/=/, $pair);
    $value =~ tr/+/ /;
    $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
    $FORM{$name} = $value;
}

if( exists( $FORM{'lat'} ) && exists( $FORM{'lon'} ) ) {
    &write_latlon;
}
else {
    &read_latlon;
}

&print_html;
exit 0;


sub read_latlon {
    $res_mode = "ReadMode";
    my $count_retry = 0;
    my $last_line;
    while ($count_retry < $conf_fopenRetry) {
        open my $file, '<', $dataFile;
        if($file) {
            while(<$file>) {
                $last_line = $_;
            }
            close $file;
            last;
        } else {
            $count_retry++;
            sleep($conf_retrySleep);
        }
    }
    if($count_retry >= $conf_fopenRetry) {
        return 1;
    }
    chop $last_line;
    ( $FORM{'lat'}, $FORM{'lon'} ) = split(/,/, $last_line);
    return 0;
}


sub write_latlon {
    $res_mode = "WriteMode";
    my $count_retry = 0;
    while ($count_retry < $conf_fopenRetry) {
        open my $file, '>', $dataFile;
        if( $file ) {
            print $file $FORM{'lat'} . "," . $FORM{'lon'} . "\n";
            close $file;
            last;
        } else {
            $conf_fopenRetry++;
            sleep($conf_retrySleep);
        }
    }

    if($count_retry >= $conf_fopenRetry) {
        return 1;
    }
    return 0;
}


sub print_html {
    print "Content-type: text/xml;\n\n";
    print <<"_HTML_";
<?xml version="1.0" encoding="utf-8" ?>
<items>
<mode>$res_mode</mode>
<error>$!</error>
<lat>$FORM{'lat'}</lat>
<lon>$FORM{'lon'}</lon>
</items>
_HTML_
}